#pragma once
#include "../parameters.hh"
#include "../math/complex.hh"
#include <iostream>
#include <cstdlib>

// Writing C_ij = (1 + as W1 + as^2 W2 + as^3 W3) * (delta_ig delta_jg + as Cp1 + as^2 Cp2 + as^3 Cp3)
// these are the Cp_n terms


// order as
extern double C1reg_pointlike_gg (double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C1reg_pointlike_qg (double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C1reg_pointlike_qqb(double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C1reg_pointlike_gg_softexp(double z, int nf, int truncation=37);
extern double C1reg_pointlike_gg_smallx (double z, int nf, int truncation=37);


// order as^2
extern double C2reg_pointlike_gg (double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C2reg_pointlike_qg (double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C2reg_pointlike_qqb(double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C2reg_pointlike_qq (double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C2reg_pointlike_qqp(double z, int nf, double muf_mu_ratio=1., double mur_muf_ratio=1.);
extern double C2reg_pointlike_gg_softexp (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qg_softexp (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qqb_softexp(double z, int nf, int truncation=37);
extern double C2reg_pointlike_qq_softexp (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qqp_softexp(double z, int nf, int truncation=37);
extern double C2reg_pointlike_gg_smallx (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qg_smallx (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qqb_smallx(double z, int nf, int truncation=37);
extern double C2reg_pointlike_qq_smallx (double z, int nf, int truncation=37);
extern double C2reg_pointlike_qqp_smallx(double z, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_gg_softexp (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qg_softexp (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qqb_softexp(dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qq_softexp (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qqp_softexp(dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_gg_smallx (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qg_smallx (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qqb_smallx(dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qq_smallx (dcomplex N, int nf, int truncation=37);
extern dcomplex C2reg_pointlike_qqp_smallx(dcomplex N, int nf, int truncation=37);
extern dcomplex C2_pointlike_gg (dcomplex N);
extern dcomplex C2_pointlike_qg (dcomplex N);
extern dcomplex C2_pointlike_qqb(dcomplex N);
extern dcomplex C2_pointlike_qq (dcomplex N);
extern dcomplex C2_pointlike_qqp(dcomplex N);


// order as^3
extern double C3reg_pointlike_gg (double z, int nf, int truncation=37, bool exact=false, double power=0);
extern double C3reg_pointlike_qg (double z, int nf, int truncation=37, bool exact=false);
extern double C3reg_pointlike_qqb(double z, int nf, int truncation=37, bool exact=false);
extern double C3reg_pointlike_qq (double z, int nf, int truncation=37, bool exact=false);
extern double C3reg_pointlike_qqp(double z, int nf, int truncation=37, bool exact=false);
//
extern double C3reg_pointlike_gg_smallx(double z, int truncation=37);



template <class T>
T C1log_pointlike_gg(T z, T D(int,T), int nf, double muf_mu_ratio=1.) {
  double lf = -2.*log(muf_mu_ratio);
  return 4.*CA*D(1,z) + 2*CA*D(0,z)*lf;
}
template <class T>
T C2log_pointlike_gg(T z, T D(int,T), int nf, double muf_mu_ratio=1.) {
  double lf = -2.*log(muf_mu_ratio);
  return ( 72. * D(3,z)
	   + (2.*nf - 33.) * D(2,z)
	   + (67. - 90.*ZETA2 - 10./3.*nf) * D(1,z)
	   + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + nf*(14./9.-2.*ZETA2)) * D(0,z)
	   )
    +    ( 108*lf * D(2,z)
	   + (36.*lf*lf - (33.-2.*nf)*lf) * D(1,z)
	   + (-(33.-2.*nf)/4.*lf*lf + (67./2.-5./3.*nf-5.*CA*CA*ZETA2)*lf) * D(0,z) );
}
template <class T>
T C3log_pointlike_gg(T z, T D(int,T), int nf, double muf_mu_ratio=1.) {
  double lf = -2.*log(muf_mu_ratio);
  if(lf!=0) { std::cout << "Error! not yet implemented!" << std::endl; std::exit(9);}
  double CA3 = CA*CA*CA;
  T Clog = ( 8*CA3 * D(5,z)
	     + (-110./9.*CA3 + 20/9.*CA*CA*nf) * D(4,z)
	     + (CA3*(925./27. - 56*ZETA2) - 164./27.*CA*CA*nf + 4./27.*CA*nf*nf) * D(3,z)
	     + (CA3 * (-1051./27. + 187./3*ZETA2 + 181*ZETA3)
		+ CA*CA*nf * (457./2./27. - 34./3.*ZETA2)
		+ 0.5*CA*CF*nf - 10./27.*CA*nf*nf ) * D(2,z)
	     + (CA3 * (30569./8./81. - 152./3.*ZETA2 - 352./3.*ZETA3 - 154./5.*ZETA2*ZETA2)
		+ CA*CA*nf * (-4211./4./81. + 94./9.*ZETA2 + 46./3.*ZETA3)
		- CA*CF*nf * (63./8.-6.*ZETA3)
		+ CA*nf*nf * (25./81.-4./9.*ZETA2) ) * D(1,z)
	     + (CA3 * (-297029./32./729. + 8563./4./81.*ZETA2 + 8941./4./27.*ZETA3 + 253./4./15.*ZETA2*ZETA2
		       - 725./2./3.*ZETA2*ZETA3 + 186.*ZETA5)
		+ CA*CA*nf * (31313./16./729. - 2173./4./81.*ZETA2 - 475./4./9.*ZETA3 - 17./2./15*ZETA2*ZETA2)
		+ CA*CF*nf * (1711./32./27. - 0.5*ZETA2 - 19./2./9.*ZETA3 - 1./5.*ZETA2*ZETA2)
		- CA*nf*nf * (58./729. - 10./27.*ZETA2 - 5./27.*ZETA3) ) * D(0,z)
	     );
  /*
  Clog += ( (16.*CA*CA*CA*lf*lf + 0*lf) * D(3,z)
	    + (4.*CA*CA*CA*lf*lf*lf - 18.*CA*CA*b0*lf*lf + 12.*CA*A1*lf) * D(2,z)
	    + (-4.*CA*CA*b0*lf*lf*lf + (8.*CA*A1+4.*CA*b0*b0-32.*CA*CA*CA*ZETA2)*lf*lf + (8*CA*B1-12*CA*b1)*lf) * D(1,z)
	    + ((2.*b0*b0-4.*CA*CA*CA*ZETA2)*lf*lf*lf + (CA*(4.*B1-5.*b1)-2.*A1*b0+6.*CA*CA*b0*ZETA2+32.*CA*CA*CA*ZETA3)*lf*lf
	       + (2.*A2-4.*CA*A1*ZETA2)*lf) * D(0,z) );
  // adding contributions from interference with C2
  T D0D1 = 3./2.*D(2,z) - ZETA2*D(0,z);
  T D0D2 = 4./3.*D(3,z) - 2.*ZETA2*D(1,z) + 2.*ZETA3*D(0,z);
  T D0D3 = 5./4.*D(4,z) - 3.*ZETA2*D(2,z) + 6.*ZETA3*D(1,z) - 6.*ZETA4*D(0,z);
  Clog += lf*(2.*CA*( 72. * D0D3(z,D)
		      + (2.*nf - 33.) * D0D2(z,D)
		      + (67. - 90.*ZETA2 - 10./3.*nf) * D0D1(z,D)
		      + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + nf*(14./9.-2.*ZETA2)) * 2*D(1,z)
		      + (_cd2 ) * D(0,z)
		      )
	      -2.*b0*( 72. * D(3,z)
		       + (2.*nf - 33.) * D(2,z)
		       + (67. - 90.*ZETA2 - 10./3.*nf) * D(1,z)
		       + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + nf*(14./9.-2.*ZETA2)) * D(0,z)
		       ) );
  */
  return Clog;
}


